#!/usr/bin/env bash

cd "$(dirname "$0")"

# turn on verbose debugging output for logs.
exec 4>&1; export BASH_XTRACEFD=4; set -x
# make errors fatal
set -e
# bleat on references to undefined shell variables
set -u

ZLIB_SOURCE_DIR="zlib"

top="$(pwd)"
stage="$top"/stage

# load autobuild provided shell functions and variables
case "$AUTOBUILD_PLATFORM" in
    windows*)
        autobuild="$(cygpath -u "$AUTOBUILD")"
    ;;
    *)
        autobuild="$AUTOBUILD"
    ;;
esac
source_environment_tempfile="$stage/source_environment.sh"
"$autobuild" source_environment > "$source_environment_tempfile"
. "$source_environment_tempfile"

VERSION_HEADER_FILE="$ZLIB_SOURCE_DIR/zlib.h"
version=$(sed -n -E 's/#define ZLIB_VERSION "([0-9.]+)"/\1/p' "${VERSION_HEADER_FILE}")
build=${AUTOBUILD_BUILD_ID:=0}
echo "${version}.${build}" > "${stage}/VERSION.txt"

pushd "$ZLIB_SOURCE_DIR"
    case "$AUTOBUILD_PLATFORM" in

        # ------------------------ windows, windows64 ------------------------
        windows*)
            load_vsvars
            mkdir -p "$stage/include/zlib"
            mkdir -p "$stage/lib/debug"
            mkdir -p "$stage/lib/release"

            mkdir -p "build"
            pushd "build"
                # Invoke cmake and use as official build
                cmake -G "$AUTOBUILD_WIN_CMAKE_GEN" -A "$AUTOBUILD_WIN_VSPLATFORM" -T host="$AUTOBUILD_WIN_VSHOST" ..

                cmake --build . --config Debug --clean-first
                cmake --build . --config Release --clean-first

                # conditionally run unit tests
                if [ "${DISABLE_UNIT_TESTS:-0}" = "0" ]; then
                    ctest -C Debug
                    ctest -C Release
                fi

                cp -a "Debug/zlibstaticd.lib" \
                    "$stage/lib/debug/zlibd.lib"
                cp -a "Debug/minizipd.lib" \
                    "$stage/lib/debug/minizip.lib"
                cp -a "Release/zlibstatic.lib" \
                    "$stage/lib/release/zlib.lib"
                cp -a "Release/minizip.lib" \
                    "$stage/lib/release/minizip.lib"
                cp -a zconf.h "$stage/include/zlib"
            popd
            cp -a zlib.h "$stage/include/zlib"
        ;;

        # ------------------------- darwin, darwin64 -------------------------
        darwin*)
            mkdir -p "$stage/include/zlib"
            mkdir -p "$stage/lib/debug"
            mkdir -p "$stage/lib/release"

            mkdir -p "build"
            pushd "build"
                # Invoke cmake and use as official build
                CFLAGS="${LL_BUILD_RELEASE}" \
                cmake -G "Xcode" -DCMAKE_OSX_SYSROOT="macosx10.14" -DCMAKE_OSX_DEPLOYMENT_TARGET="10.13" ..

                cmake --build . --config Debug --clean-first
                cmake --build . --config Release --clean-first

                # conditionally run unit tests
                if [ "${DISABLE_UNIT_TESTS:-0}" = "0" ]; then
                    ctest -C Debug
                    ctest -C Release
                fi

                pushd "Debug"
                	fix_dylib_id "libz.dylib"
                	cp -a "libminizip.a" "$stage/lib/debug"
                	cp -a "libz.a" "$stage/lib/debug"
                	cp -a libz*.dylib "$stage/lib/debug"
                popd

                pushd "Release"
                    fix_dylib_id "libz.dylib"
                	cp -a "libminizip.a" "$stage/lib/release"
                	cp -a "libz.a" "$stage/lib/release"
                	cp -a libz*.dylib "$stage/lib/release"
                popd
                cp -a zconf.h "$stage/include/zlib"
            popd
            cp -a zlib.h "$stage/include/zlib"
        ;;            

        # -------------------------- linux, linux64 --------------------------
        linux*)
            # Linux build environment at Linden comes pre-polluted with stuff that can
            # seriously damage 3rd-party builds.  Environmental garbage you can expect
            # includes:
            #
            #    DISTCC_POTENTIAL_HOSTS     arch           root        CXXFLAGS
            #    DISTCC_LOCATION            top            branch      CC
            #    DISTCC_HOSTS               build_name     suffix      CXX
            #    LSDISTCC_ARGS              repo           prefix      CFLAGS
            #    cxx_version                AUTOBUILD      SIGN        CPPFLAGS
            #
            # So, clear out bits that shouldn't affect our configure-directed build
            # but which do nonetheless.
            #
            unset DISTCC_HOSTS CC CXX CFLAGS CPPFLAGS CXXFLAGS

            # Default target per autobuild build --address-size
            opts="${TARGET_OPTS:--m$AUTOBUILD_ADDRSIZE}"

            # Handle any deliberate platform targeting
            if [ ! "${TARGET_CPPFLAGS:-}" ]; then
                # Remove sysroot contamination from build environment
                unset CPPFLAGS
            else
                # Incorporate special pre-processing flags
                export CPPFLAGS="$TARGET_CPPFLAGS"
            fi

            mkdir -p "$stage/include/zlib"
            mkdir -p "$stage/lib/debug"
            mkdir -p "$stage/lib/release"

            mkdir -p "build_release"
            pushd "build_release"
                # Invoke cmake and use as official build
                CFLAGS="$opts -g -fPIC -DPIC -fstack-protector-strong -D_FORTIFY_SOURCE=2" CXXFLAGS="$opts" \
                    cmake -DCMAKE_BUILD_TYPE="Release" ..

                cmake --build . --config Release --clean-first

                # conditionally run unit tests
                if [ "${DISABLE_UNIT_TESTS:-0}" = "0" ]; then
                    ctest -C Release
                fi

                cp -a "libminizip.a" "$stage/lib/release"
                cp -a "libz.a" "$stage/lib/release"
                cp -a libz*.so* "$stage/lib/release"
                cp -a zconf.h "$stage/include/zlib"
            popd

            mkdir -p "build_debug"
            pushd "build_debug"
                # Invoke cmake and use as official build
                CFLAGS="$opts -Og -g -fPIC -DPIC" CXXFLAGS="$opts" \
                    cmake -DCMAKE_BUILD_TYPE="Debug" ..

                cmake --build . --config Debug --clean-first

                # conditionally run unit tests
                if [ "${DISABLE_UNIT_TESTS:-0}" = "0" ]; then
                    ctest -C Debug
                fi

                cp -a "libminizip.a" "$stage/lib/debug"
                cp -a "libz.a" "$stage/lib/debug"
                cp -a libz*.so* "$stage/lib/debug"
            popd

            cp -a zlib.h "$stage/include/zlib"
        ;;
    esac
    mkdir -p "$stage/LICENSES"
    # The copyright info for zlib is the tail end of its README file. Tempting
    # though it is to write something like 'tail -n 31 README', that will
    # quietly fail if the length of the copyright notice ever changes.
    # Instead, look for the section header that sets off that passage and copy
    # from there through EOF. (Given that END is recognized by awk, you might
    # reasonably expect '/pattern/,END' to work, but no: END can only be used
    # to fire an action past EOF. Have to simulate by using another regexp we
    # hope will NOT match.)
    awk '/^Copyright notice:$/,/@%rest%@/' README > "$stage/LICENSES/zlib.txt"
    # In case the section header changes, ensure that zlib.txt is non-empty.
    # (With -e in effect, a raw test command has the force of an assert.)
    # Exiting here means we failed to match the copyright section header.
    # Check the README and adjust the awk regexp accordingly.
    [ -s "$stage/LICENSES/zlib.txt" ]
    pushd contrib/minizip
        mkdir -p "$stage"/include/minizip/
        cp -a ioapi.h zip.h unzip.h "$stage"/include/minizip/
        awk '/^License$/,/@%rest%@/' MiniZip64_info.txt > "$stage/LICENSES/minizip.txt"
        [ -s "$stage/LICENSES/minizip.txt" ]
    popd
popd

mkdir -p "$stage"/docs/zlib/
cp -a README.Linden "$stage"/docs/zlib/
